// // /*
// //  * Copyright (c) 2016 Intel Corporation
// //  *
// //  * SPDX-License-Identifier: Apache-2.0
// //  */

// #include <stdio.h>
// #include <zephyr/kernel.h>
// #include <zephyr/drivers/gpio.h>

// /* 1000 msec = 1 sec */
// #define SLEEP_TIME_MS   1000

// /* The devicetree node identifier for the "led0" alias. */
// #define LED0_NODE DT_ALIAS(led0)

// /*
//  * A build error on this line means your board is unsupported.
//  * See the sample documentation for information on how to fix this.
//  */
// static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

// int main(void)
// {
// 	int ret;
// 	bool led_state = true;

// 	if (!gpio_is_ready_dt(&led)) {
// 		return 0;
// 	}

// 	ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
// 	if (ret < 0) {
// 		return 0;
// 	}

// 	while (1) {
// 		ret = gpio_pin_toggle_dt(&led);
// 		if (ret < 0) {
// 			return 0;
// 		}

// 		led_state = !led_state;
// 		printf("LED state: %s\n", led_state ? "ON" : "OFF");
// 		k_msleep(SLEEP_TIME_MS);
// 	}
// 	return 0;
// }

#include <zephyr/kernel.h>
    #include <zephyr/device.h>
    #include <zephyr/drivers/spi.h>
    #include <zephyr/fs/fs.h>
    #include <zephyr/storage/disk_access.h>
    #include <zephyr/logging/log.h>
    
    LOG_MODULE_REGISTER(main, LOG_LEVEL_DBG);
    
    #define MOUNT_POINT "/SD"
    #define FILE_PATH MOUNT_POINT "/arduino.txt"
    
    static struct fs_mount_t fs_mount_sd = {
        .type = FS_FATFS,
        .mnt_point = MOUNT_POINT,
        .fs_data = NULL,
        .storage_dev = (void *)"SD",
    };
    
    bool sd_initialized = false;
    
    // Function to initialize SPI device
    const struct device *init_spi_device() {
        const struct device *spi_dev = DEVICE_DT_GET(DT_NODELABEL(spi2));
        if (!spi_dev) {
            printk("SPI device not found!\n");
        } else {
            printk("SPI device found: %p\n", spi_dev);
            printk("SPI device name: %s\n", spi_dev->name);
        }
        return spi_dev;
    }
    
    // Initialize SD card
    int initialize_sd_card() {
        printk("Attempting to initialize SD card...\n");
    
        // Check SPI device availability
        const struct device *spi_dev = init_spi_device();
        if (!spi_dev) {
            printk("Cannot proceed without SPI device!\n");
            sd_initialized = false;
            return -ENODEV;
        }
    
        int ret = disk_access_init("SD");
        if (ret != 0) {
            printk("SD card initialization failed. Error: %d\n", ret);
            LOG_WRN("SD card initialization failed. Card may be removed.");
            sd_initialized = false;
            return ret;
        }
    
        printk("SD card initialization successful.\n");
    
        ret = fs_mount(&fs_mount_sd);
        if (ret == 0) {
            printk("SD card mounted successfully at %s.\n", MOUNT_POINT);
            LOG_INF("SD card initialized successfully.");
            sd_initialized = true;
            return 0;
        } else {
            printk("Failed to mount SD card. Error: %d\n", ret);
            LOG_ERR("Failed to mount SD card: %d", ret);
            sd_initialized = false;
            return ret;
        }
    }
    
    // Write to SD card
    void write_string_to_file(const char *str) {
        printk("Preparing to write to file: %s\n", FILE_PATH);
    
        struct fs_file_t file;
        fs_file_t_init(&file);
    
        int ret = fs_open(&file, FILE_PATH, FS_O_WRITE | FS_O_CREATE | FS_O_APPEND);
        if (ret < 0) {
            printk("Failed to open arduino.txt for writing. Error: %d\n", ret);
            LOG_ERR("Failed to open arduino.txt for writing: %d", ret);
            sd_initialized = false;
            return;
        }
    
        printk("File opened successfully. Writing data...\n");
    
        ret = fs_write(&file, str, strlen(str));
        if (ret < 0) {
            printk("Failed to write to arduino.txt. Error: %d\n", ret);
            LOG_ERR("Failed to write to arduino.txt: %d", ret);
        } else {
            printk("Data written successfully to arduino.txt: %s\n", str);
            LOG_INF("Written to arduino.txt: %s", str);
        }
    
        fs_close(&file);
        printk("File closed.\n");
    }
    
    void main(void) {
        printk("Starting SD card test with SPI on nRF9160 DK...\n");
    
        if (initialize_sd_card() == 0) {
            write_string_to_file("Hello, SPI MicroSD!");
        }
    
        while (1) {
            if (sd_initialized) {
                printk("SD card is initialized. Writing data...\n");
                write_string_to_file("Hello from Zephyr over SPI!");
            } else {
                printk("SD card not initialized. Retrying...\n");
                LOG_WRN("SD card not initialized. Retrying...");
                initialize_sd_card();
            }
            k_sleep(K_SECONDS(5));
        }
    }